﻿using System;
using System.Collections.Generic;
using System.Drawing;

using DevExpress.XtraEditors;
using DevExpress.XtraRichEdit;
using DevExpress.XtraRichEdit.API.Native;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : XtraForm
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 줄 문자열
        /// </summary>
        private const string LINE_STRING = "----------------------------------------------------------------------------------------------------";

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.richEditControl.ActiveViewType = RichEditViewType.Simple;

            this.richEditControl.Options.HorizontalScrollbar.Visibility = RichEditScrollbarVisibility.Hidden;
            this.richEditControl.Options.VerticalScrollbar.Visibility   = RichEditScrollbarVisibility.Auto;

            this.richEditControl.Document.DefaultCharacterProperties.FontName  = "나눔고딕코딩";
            this.richEditControl.Document.DefaultCharacterProperties.FontSize  = 12f;
            this.richEditControl.Document.DefaultCharacterProperties.ForeColor = Color.Black;

            this.richEditControl.Document.ParagraphStyles[0].FontName  = "나눔고딕코딩";
            this.richEditControl.Document.ParagraphStyles[0].FontSize  = 12f; 
            this.richEditControl.Document.ParagraphStyles[0].ForeColor = Color.Black;

            this.richEditControl.LoadDocument("sample.rtf");

            this.parseButton.Click += parseButton_Click;
        }

        #endregion


        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 분석 버튼 클릭시 처리하기 - parseButton_Click(sender, e)

        /// <summary>
        /// 분석 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void parseButton_Click(object sender, EventArgs e)
        {
            Document document = this.richEditControl.Document;

            while(document.Tables.Count > 0)
            {
                PasteTableContent(document, document.Tables.Count - 1);

                document.Tables.RemoveTableAt(document.Tables.Count - 1);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 라인 분리자 문단 리스트 구하기 - GetLineSeparatorParagraphList(document)

        /// <summary>
        /// 라인 분리자 문단 리스트 구하기
        /// </summary>
        /// <param name="document">문서</param>
        /// <returns>라인 분리자 문단 리스트</returns>
        private List<Paragraph> GetLineSeparatorParagraphList(Document document)
        {
            List<Paragraph> list = new List<Paragraph>();

            for(int i = 0; i < document.Paragraphs.Count - 1; i++)
            {
                Paragraph paragraph1 = document.Paragraphs[i    ];
                Paragraph paragraph2 = document.Paragraphs[i + 1];

                string text1 = document.GetText(paragraph1.Range);
                string text2 = document.GetText(paragraph2.Range);

                if(text1 == LINE_STRING && text2 == LINE_STRING)
                {
                    list.Add(paragraph1);
                }
            }

            return list;
        }

        #endregion
        #region 테이블 내용 붙여넣기 - PasteTableContent(document, index)

        /// <summary>
        /// 테이블 내용 붙여넣기
        /// </summary>
        /// <param name="document">문서</param>
        /// <param name="index">인덱스</param>
        private void PasteTableContent(Document document, int index)
        {
            List<Paragraph> list = GetLineSeparatorParagraphList(document);

            Table table = document.Tables[index];

            DocumentRange contentRange = table.Cell(0, 0).ContentRange;

            DocumentRange range = document.CreateRange(contentRange.Start.ToInt() + 1, contentRange.End.ToInt() - contentRange.Start.ToInt() + 1 - 2);

            document.Copy(range);

            document.CaretPosition = document.CreatePosition(list[index].Range.End.ToInt());

            document.Paste();
        }

        #endregion
    }
}